/**
 *  qdsdl.h by Tyler Montbriand, 2004
 *  Free software as in free beer.  Do whatever you want with it..
 */

/**
 *   qdsdl -- a simple library of inline C++ header files to simplify 
 * the functionality of SDL and a few of it's expansion libraries.
 *
 *   The idea of qdsdl is NOT to wrap the entire functionality of SDL.  
 * That would be a huge undertaking and kind of pointless -- the SDL 
 * functionality is pretty damn good in the first place, why reinvent it 
 * with new wrappers that you must use in a totally different way?
 *
 *   Instead, qdsdl wrapper classes are guaranteed to be interchangable 
 * with the SDL constructs they wrap.  You don't need to relearn how to 
 * use SDL  from a new point of view.  For instance, once you create 
 * a `qdmutex', you can call SDL_mutexP and SDL_mutexV on it just like 
 * it's a `SDL_Mutex *'.
 *
 *   The difference between a `qdmutex' and a `SDL_Mutex *' is just
 * sensible default values and garbage collection.  The constructor 
 * calls SDL_CreateMutex for you when it goes into scope, and 
 * SDL_DestroyMutex for you when it goes out of scope.
 *
 *   This is not an add-on library to SDL.  It is 100% inline.  This 
 * means  there are no qdsdl binaries to link in, and it does not 
 * require any libraries other than the ones you'd be using in the first 
 * place.  It adds no bloat to your code, this is stuff you'd be doing 
 * anyway -- it just makes it more convenient.
 *
 *   This is what the library includes so far:
 *
 * qdsdl.h    SDL initialization and deinitialization
 * qdnet.h    SDLNet initialization and deinitialization
 * qdthread.h Mutexes, semaphores, conds, and threads
 * qdtime.h   Stopwatches and timer callbacks
 * qdsock.h   TCP sockets, UDP sockets, UDP packets,
 *            IP addresses, and socket sets
 * qdswap.h   16-bit and 32-bit swap operators guaranteed to work 
 *            at compile-time
 * qdwrap.h   a template wrapper class to simplify making 
 *            quick-and-dirty wrappers of pointers to something...  
 *            nearly everything in qdsdl uses it.
 */
#ifndef __QDSDL_H__
#define __QDSDL_H__

#ifndef __cplusplus
#error "qdsdl.h needs c++ compilation"
#endif

#include <stdio.h>
#include <SDL/SDL.h>

/**
 * Quick-and-dirty class to initialize and quit SDL.
 * Just plunk down a qdsdl object somewhere, and SDL will
 * initialize when it goes into scope and quit when it goes out of 
 * scope.  Should be nestable, since SDL_Init/SDL_Quit calls are
 * supposed to be nestable.
 *
 * Example:
 *   #include <stdio.h>
 *   #include "qdsdl.h"
 *   int main()
 *   {
 *     qdsdl sdl;
 *
 *     if(!sdl.Good())
 *       return(1);
 *
 *     fprintf(stderr,"SDL initialized successfully\n");
 *     return(0);
 *   }
 */
class qdsdl
{
public:
  inline qdsdl(Uint32 flag):good(false)  { Init(flag); }
  inline ~qdsdl()                        { Quit();     }

  inline Uint32 WasInit(Uint32 flag=SDL_INIT_EVERYTHING)
  { return(SDL_WasInit(flag)); }

  inline void QuitSubSystem(Uint32 flag) { SDL_QuitSubSystem(flag); }
  inline bool InitSubSystem(Uint32 flag)
  {
    if(SDL_InitSubSystem(flag)<0)
    {
      fprintf(stderr,"Couldn't init subsystems 0x%08x:%s\n",
        flag,SDL_GetError());
      return(false);
    }

    return(true);
  }

  inline bool Init(Uint32 flag=0)
  {
    if(SDL_Init(flag)<0)
    {
      fprintf(stderr,"Couldn't initialize SDL: %s\n",SDL_GetError());
      return(false);
    }
    good=true;
    return(true);
  }

  inline void Quit() { if(good) { SDL_Quit(); good=false; } }

  inline bool Good() { return good; }
private:
  bool good;
};

#endif/*__QDSDL_H__*/
