/**
 *  qdthread.h by Tyler Montbriand, 2004
 *  Free software as in free beer.  Do whatever you want with it..
 */

/*******************************************************************
     This header file contains a number of useful classes for dealing 
   with mutexes, semaphores, and threads.  Note that SDL_Init should
   be called before you use these structures.
     qdsem and qdmutex behave precisely like SDL_sem *'s and SDL_mutex *'s
   respectively but they create and destroy the mutex and semaphore for 
   you when the classes come into and go out of scope.
     qdthread makes it easier to create threads.  It SDL_KillThread's 
   the thread when it exits, so if you want to wait for exit call 
   thread.wait() before it goes out of scope.
 *******************************************************************/
#ifndef __QDTHREAD_H__
#define __QDTHREAD_H__

#ifndef __cplusplus
#error "qdthread.h needs c++ compilation"
#endif

#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>

#include "qdwrap.h"

/**
 * Extremely simple and useful wrapper for semaphores.  Allocates and 
 * destroys for you when it enters and leaves scope.  Can be used
 * by SDL semaphore functions just like a plain SDL_sem *.
 * Example:
 *
 *  #include <stdio.h>
 *  #include "qdsdl.h"
 *  #include "qdthread.h"
 *  int main(int argc, char *argv[])
 *  {
 *    qdsdl sdl; // Initialize SDL
 *    if(!sdl.Good())  return(1);
 *    {
 *      qdsem sem(0); // Create semaphore with value 0
 *      fprintf(stderr,"sem == %d\n",SDL_SemValue(sem)); // prints sem == 0
 *      SDL_SemPost(sem);
 *      fprintf(stderr,"sem == %d\n",SDL_SemValue(sem)); // prints sem == 1
 *    } // sem goes out of scope and is freed
 *  } // sdl goes out of scope, is deinitialized
 */
class qdsem:public qdwrap<SDL_sem>
{
public:
  inline qdsem(int ival):qdwrap<SDL_sem>(SDL_CreateSemaphore(ival)) { }
  inline ~qdsem()
  {
    if(val!=NULL) // Error tolerant!
    {
      SDL_DestroySemaphore(val);
      val=NULL;
    }
  }
};

/**
 * Extremely simple and useful wrapper for mutexes.  Creates and 
 * destroys for you when it enters and leaves scope.  Usable by SDL
 * mutex functions just like a plain SDL_mutex *.
 *
 * Example:
 *
 *  #include "qdsdl.h"
 *  #include "qdthread.h"
 *
 *  int     resource=0;  // Resource to protect against race conditions
 *  qdmutex m;           // Mutex you must lock to use resource
 *
 *  int somefunction()
 *  {
 *    SDL_mutexP(lock);
 *    resource++;
 *    SDL_mutexV(lock);
 *    return(0);
 *  }
 */
class qdmutex:public qdwrap<SDL_mutex>
{
public:
  inline qdmutex():qdwrap<SDL_mutex>(SDL_CreateMutex()) { }
  inline ~qdmutex()
  {
    if(val!=NULL) // Error tolerant!
    {
      SDL_DestroyMutex(val);
      val=NULL;
    }
  }
};

/**
 * Extremely simple wrapper for SDL threads.  Can launch threads for you
 * when it goes into scope, and automatically kills threads when it 
 * goes out of scope.  Use Wait() if you want a more graceful way of
 * stopping the thread, as per SDL docs for SDL_WaitThread.  Does NOT 
 * create more than one thread if you call start() repeatedly.
 * Example:
 *
 *  #include "qdsdl.h"
 *  #include "qdthread.h"
 *
 *  int function(void *userdata)
 *  {
 *    fprintf(stderr,"Survey says, %s\n",userdata);
 *    return(0);    
 *  }
 *
 *  int main()
 *  {
 *    qdsdl sdl; // Initialize SDL
 *    if(!sdl.Good())
 *      return(1);
 *
 *    { // Want thread to be in a higher scope, so it gets freed first
 *      qdthread thread(function,"You Farted!");
 *      fprintf(stderr,"Thread returned %d\n",thread.wait());
 *    } // Thread goes out of scope before sdl
 *    return(0);
 *  } // sdl goes out of scope, quits SDL
 */
class qdthread:public qdwrap<SDL_Thread>
{
public:
  inline qdthread():qdwrap<SDL_Thread>(NULL) { }

  inline qdthread(int (*fn)(void *), void *dat=NULL):qdwrap<SDL_Thread>(NULL)
  {
    start(fn,dat);
  }

  inline bool start(int (*fn)(void *), void *dat=NULL)
  {
    stop();
    val=SDL_CreateThread(fn,dat);
    return(val!=NULL);
  }

  inline int wait()
  {
    int s;
    if(val==NULL)
      return(0);

    SDL_WaitThread(val,&s);
    val=NULL;
    return(s);    
  }

  inline void stop()
  {
    if(val!=NULL)
    {
      SDL_KillThread(val);
      val=NULL;
    }
  }

  inline ~qdthread() { stop(); }
private:
};

/**
 * Brutally simple and useful wrapper for SDL_cond's.  Creates a cond
 * when it goes into scope and destroys it when it goes out of scope.
 * A full explanation of conditional variables is too big to fit here, 
 * so see SDL_zig( http://burningsmell.dyndns.org/sdl_zig/index.html )
 * for a better explanation.
 */
class qdcond:public qdwrap<SDL_cond>
{
public:
  inline qdcond():qdwrap<SDL_cond>(NULL) { val=SDL_CreateCond(); }
  inline ~qdcond() { if(val!=NULL) { SDL_DestroyCond(val); val=NULL; } }  
};

#endif/*__QDTHREAD_H__*/
